@@ -16,27 +16,32 @@
  * Boston, MA 021110-1307, USA.
  */
 
+#include "kerncompat.h"
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <dirent.h>
-#include <uuid/uuid.h>
 #include <getopt.h>
-
-#include "kerncompat.h"
+#include <errno.h>
+#include <stdbool.h>
+#include <string.h>
+#include <uuid/uuid.h>
 #include "kernel-shared/ctree.h"
 #include "kernel-shared/disk-io.h"
 #include "kernel-shared/transaction.h"
-#include "common/utils.h"
 #include "kernel-shared/volumes.h"
+#include "kernel-shared/extent_io.h"
+#include "common/defs.h"
+#include "common/utils.h"
+#include "common/extent-cache.h"
 #include "common/open-utils.h"
 #include "common/parse-utils.h"
 #include "common/device-scan.h"
+#include "common/messages.h"
+#include "common/string-utils.h"
 #include "common/help.h"
 #include "common/box.h"
+#include "ioctl.h"
 
 static char *device;
 static int force = 0;
@@ -120,13 +125,12 @@ static int set_metadata_uuid(struct btrfs_root *root, const char *uuid_string)
 	uuid_changed = incompat_flags & BTRFS_FEATURE_INCOMPAT_METADATA_UUID;
 
 	if (super_flags & BTRFS_SUPER_FLAG_SEEDING) {
-		fprintf(stderr, "cannot set metadata UUID on a seed device\n");
+		error("cannot set metadata UUID on a seed device");
 		return 1;
 	}
 
 	if (check_unfinished_fsid_change(root->fs_info, unused1, unused2)) {
-		fprintf(stderr,
-			"UUID rewrite in progress, cannot change fsid\n");
+		error("UUID rewrite in progress, cannot change fsid");
 		return 1;
 	}
 
@@ -442,8 +446,7 @@ static int rewrite_checksums(struct btrfs_root *root, int csum_type)
 
 	/* FIXME: Sanity checks */
 	if (0) {
-		fprintf(stderr,
-			"UUID rewrite in progress, cannot change fsid\n");
+		error("UUID rewrite in progress, cannot change fsid");
 		return 1;
 	}
 
@@ -775,12 +778,138 @@ out:
 	return ret;
 }
 
+/* After this many block groups we need to commit transaction. */
+#define BLOCK_GROUP_BATCH	64
+
+static int convert_to_bg_tree(struct btrfs_fs_info *fs_info)
+{
+	struct btrfs_super_block *sb = fs_info->super_copy;
+	struct btrfs_trans_handle *trans;
+	struct cache_extent *ce;
+	int converted_bgs = 0;
+	int ret;
+
+	trans = btrfs_start_transaction(fs_info->tree_root, 2);
+	if (IS_ERR(trans)) {
+		ret = PTR_ERR(trans);
+		errno = -ret;
+		error_msg(ERROR_MSG_START_TRANS, "%m");
+		return ret;
+	}
+
+	/* Set NO_HOLES feature */
+	btrfs_set_super_incompat_flags(sb, btrfs_super_incompat_flags(sb) |
+				       BTRFS_FEATURE_INCOMPAT_NO_HOLES);
+
+	/* We're resuming from previous run. */
+	if (btrfs_super_flags(sb) & BTRFS_SUPER_FLAG_CHANGING_BG_TREE)
+		goto iterate_bgs;
+
+	ret = btrfs_create_root(trans, fs_info,
+				BTRFS_BLOCK_GROUP_TREE_OBJECTID);
+	if (ret < 0) {
+		error("failed to create block group root: %d", ret);
+		goto error;
+	}
+	btrfs_set_super_flags(sb,
+			btrfs_super_flags(sb) |
+			BTRFS_SUPER_FLAG_CHANGING_BG_TREE);
+	fs_info->last_converted_bg_bytenr = (u64)-1;
+
+	/* Now commit the transaction to make above changes to reach disks. */
+	ret = btrfs_commit_transaction(trans, fs_info->tree_root);
+	if (ret < 0) {
+		error_msg(ERROR_MSG_COMMIT_TRANS, "new bg root: %d", ret);
+		goto error;
+	}
+	trans = btrfs_start_transaction(fs_info->tree_root, 2);
+	if (IS_ERR(trans)) {
+		ret = PTR_ERR(trans);
+		errno = -ret;
+		error_msg(ERROR_MSG_START_TRANS, "%m");
+		return ret;
+	}
+
+iterate_bgs:
+	if (fs_info->last_converted_bg_bytenr == (u64)-1) {
+		ce = last_cache_extent(&fs_info->mapping_tree.cache_tree);
+	} else {
+		ce = search_cache_extent(&fs_info->mapping_tree.cache_tree,
+					 fs_info->last_converted_bg_bytenr);
+		if (!ce) {
+			error("failed to find block group for bytenr %llu",
+			      fs_info->last_converted_bg_bytenr);
+			ret = -ENOENT;
+			goto error;
+		}
+		ce = prev_cache_extent(ce);
+		if (!ce) {
+			error("no more block group before bytenr %llu",
+			      fs_info->last_converted_bg_bytenr);
+			ret = -ENOENT;
+			goto error;
+		}
+	}
+
+	/* Now convert each block */
+	while (ce) {
+		struct cache_extent *prev = prev_cache_extent(ce);
+		u64 bytenr = ce->start;
+
+		ret = btrfs_convert_one_bg(trans, bytenr);
+		if (ret < 0)
+			goto error;
+		converted_bgs++;
+		ce = prev;
+
+		if (converted_bgs % BLOCK_GROUP_BATCH == 0) {
+			ret = btrfs_commit_transaction(trans,
+							fs_info->tree_root);
+			if (ret < 0) {
+				errno = -ret;
+				error_msg(ERROR_MSG_COMMIT_TRANS, "%m");
+				return ret;
+			}
+			trans = btrfs_start_transaction(fs_info->tree_root, 2);
+			if (IS_ERR(trans)) {
+				ret = PTR_ERR(trans);
+				errno = -ret;
+				error_msg(ERROR_MSG_START_TRANS, "%m");
+				return ret;
+			}
+		}
+	}
+	/*
+	 * All bgs converted, remove the CHANGING_BG flag and set the compat ro
+	 * flag.
+	 */
+	fs_info->last_converted_bg_bytenr = 0;
+	btrfs_set_super_flags(sb,
+		btrfs_super_flags(sb) &
+		~BTRFS_SUPER_FLAG_CHANGING_BG_TREE);
+	btrfs_set_super_compat_ro_flags(sb,
+			btrfs_super_compat_ro_flags(sb) |
+			BTRFS_FEATURE_COMPAT_RO_BLOCK_GROUP_TREE);
+	ret = btrfs_commit_transaction(trans, fs_info->tree_root);
+	if (ret < 0) {
+		errno = -ret;
+		error_msg(ERROR_MSG_COMMIT_TRANS, "final transaction: %m");
+		return ret;
+	}
+	printf("Converted the filesystem to block group tree feature\n");
+	return 0;
+error:
+	btrfs_abort_transaction(trans, ret);
+	return ret;
+}
+
 static void print_usage(void)
 {
 	printf("usage: btrfstune [options] device\n");
 	printf("Tune settings of filesystem features on an unmounted device\n\n");
 	printf("Options:\n");
 	printf("  change feature status:\n");
+	printf("\t-b          enable block group tree (mkfs: block-group-tree, for less mount time)\n");
 	printf("\t-r          enable extended inode refs (mkfs: extref, for hardlink limits)\n");
 	printf("\t-x          enable skinny metadata extent refs (mkfs: skinny-metadata)\n");
 	printf("\t-n          enable no-holes feature (mkfs: no-holes, more efficient sparse file representation)\n");
@@ -811,6 +940,7 @@ int BOX_MAIN(btrfstune)(int argc, char *argv[])
 	u64 seeding_value = 0;
 	int random_fsid = 0;
 	int change_metadata_uuid = 0;
+	bool to_bg_tree = false;
 	int csum_type = -1;
 	char *new_fsid_str = NULL;
 	int ret;
@@ -818,7 +948,7 @@ int BOX_MAIN(btrfstune)(int argc, char *argv[])
 	int fd = -1;
 
 	while(1) {
-		enum { GETOPT_VAL_CSUM = 256 };
+		enum { GETOPT_VAL_CSUM = GETOPT_VAL_FIRST };
 		static const struct option long_options[] = {
 			{ "help", no_argument, NULL, GETOPT_VAL_HELP},
 #if EXPERIMENTAL
@@ -826,11 +956,18 @@ int BOX_MAIN(btrfstune)(int argc, char *argv[])
 #endif
 			{ NULL, 0, NULL, 0 }
 		};
+#if EXPERIMENTAL
+		int c = getopt_long(argc, argv, "S:rxfuU:nmM:b", long_options, NULL);
+#else
 		int c = getopt_long(argc, argv, "S:rxfuU:nmM:", long_options, NULL);
+#endif
 
 		if (c < 0)
 			break;
 		switch(c) {
+		case 'b':
+			to_bg_tree = true;
+			break;
 		case 'S':
 			seeding_flag = 1;
 			seeding_value = arg_strtou64(optarg);
@@ -890,7 +1027,7 @@ int BOX_MAIN(btrfstune)(int argc, char *argv[])
 		return 1;
 	}
 	if (!super_flags && !seeding_flag && !(random_fsid || new_fsid_str) &&
-	    !change_metadata_uuid && csum_type == -1) {
+	    !change_metadata_uuid && csum_type == -1 && !to_bg_tree) {
 		error("at least one option should be specified");
 		print_usage();
 		return 1;
@@ -936,9 +1073,27 @@ int BOX_MAIN(btrfstune)(int argc, char *argv[])
 		return 1;
 	}
 
+	if (to_bg_tree) {
+		if (btrfs_fs_compat_ro(root->fs_info, BLOCK_GROUP_TREE)) {
+			error("the filesystem already has block group tree feature");
+			ret = 1;
+			goto out;
+		}
+		if (!btrfs_fs_compat_ro(root->fs_info, FREE_SPACE_TREE_VALID)) {
+			error("the filesystem doesn't have space cache v2, needs to be mounted with \"-o space_cache=v2\" first");
+			ret = 1;
+			goto out;
+		}
+		ret = convert_to_bg_tree(root->fs_info);
+		if (ret < 0) {
+			error("failed to convert the filesystem to block group tree feature");
+			goto out;
+		}
+		goto out;
+	}
 	if (seeding_flag) {
 		if (btrfs_fs_incompat(root->fs_info, METADATA_UUID)) {
-			fprintf(stderr, "SEED flag cannot be changed on a metadata-uuid changed fs\n");
+			error("SEED flag cannot be changed on a metadata-uuid changed fs");
 			ret = 1;
 			goto out;
 		}
@@ -948,7 +1103,7 @@ int BOX_MAIN(btrfstune)(int argc, char *argv[])
 "this is dangerous, clearing the seeding flag may cause the derived device not to be mountable!");
 			ret = ask_user("We are going to clear the seeding flag, are you sure?");
 			if (!ret) {
-				fprintf(stderr, "Clear seeding flag canceled\n");
+				error("clear seeding flag canceled");
 				ret = 1;
 				goto out;
 			}
@@ -975,8 +1130,7 @@ int BOX_MAIN(btrfstune)(int argc, char *argv[])
 
 	if (change_metadata_uuid) {
 		if (seeding_flag) {
-			fprintf(stderr,
-		"Not allowed to set both seeding flag and uuid metadata\n");
+			error("not allowed to set both seeding flag and uuid metadata");
 			ret = 1;
 			goto out;
 		}
@@ -993,9 +1147,9 @@ int BOX_MAIN(btrfstune)(int argc, char *argv[])
 
 	if (random_fsid || (new_fsid_str && !change_metadata_uuid)) {
 		if (btrfs_fs_incompat(root->fs_info, METADATA_UUID)) {
-			fprintf(stderr,
+			error(
 		"Cannot rewrite fsid while METADATA_UUID flag is active. \n"
-		"Ensure fsid and metadata_uuid match before retrying.\n");
+		"Ensure fsid and metadata_uuid match before retrying.");
 			ret = 1;
 			goto out;
 		}
@@ -1007,7 +1161,7 @@ int BOX_MAIN(btrfstune)(int argc, char *argv[])
 "\tIf cancelled or interrupted, run 'btrfstune -u' to restart.");
 			ret = ask_user("We are going to change UUID, are your sure?");
 			if (!ret) {
-				fprintf(stderr, "UUID change canceled\n");
+				error("UUID change canceled");
 				ret = 1;
 				goto out;
 			}
