@@ -19,6 +19,11 @@
 #ifndef __KERNCOMPAT_H__
 #define __KERNCOMPAT_H__
 
+#ifndef __SANE_USERSPACE_TYPES__
+/* For PPC64 to get LL64 types */
+#define __SANE_USERSPACE_TYPES__
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -134,8 +139,10 @@ static inline void bugon_trace(const char *assertion, const char *filename,
 #define __bitwise__ __attribute__((bitwise))
 #else
 #define __force
+#ifndef __bitwise__
 #define __bitwise__
 #endif
+#endif
 
 #ifndef __CHECKER__
 /*
@@ -526,4 +533,21 @@ struct __una_u64 { __le64 x; } __attribute__((__packed__));
 #define noinline
 #endif
 
+/*
+ * Note: simplified versions of READ_ONCE and WRITE_ONCE for source
+ * compatibility only, not usable for lock-less implementation like in kernel.
+ *
+ * Changed:
+ * - __unqual_scalar_typeof: volatile cast to typeof()
+ * - compiletime_assert_rwonce_type: no word size compatibility checks
+ * - no const volatile cast
+ */
+
+#define READ_ONCE(x)		(x)
+
+#define WRITE_ONCE(x, val)						\
+do {									\
+	(x) = (val);							\
+} while (0)
+
 #endif
